-- Broker_MicroMenu by yess, yessica@fantasymail.de

local ldb = LibStub:GetLibrary("LibDataBroker-1.1",true)
local LibQTip = LibStub('LibQTip-1.0')

local delay = 1
local counter = 0
local dataobj, tooltip
local color = true
local path = "Interface\\AddOns\\Broker_MicroMenu\\media\\"

function Debug(...)
	--[===[@debug@
	local s = "Broker_MicroMenu Debug:"
	for i=1,select("#", ...) do
		local x = select(i, ...)
		s = strjoin(" ",s,tostring(x))
	end
	DEFAULT_CHAT_FRAME:AddMessage(s)
	--@end-debug@]===]
end

local function RGBToHex(r, g, b)
	return ("%02x%02x%02x"):format(r*255, g*255, b*255)	
end

local mb = MainMenuMicroButton:GetScript("OnMouseUp")
local function mainmenu(self, ...) self.down = 1; mb(self, ...) end

dataobj = ldb:NewDataObject("Broker_MicroMenu", {
	type = "data source",
	icon = path.."green.tga",
	label = "MicroMenu",
	text  = "0 fps 0 ms",
	OnClick = function(self, button, ...) 
		if button == "RightButton" then 
			mainmenu(self, button, ...)
		else
			ToggleCharacter("PaperDollFrame")
		end
	end
})

-------------------------
-- custom libqtip cell
-------------------------
local myProvider, cellPrototype = LibQTip:CreateCellProvider()

function cellPrototype:InitializeCell()
	self.texture = self:CreateTexture()
	self.texture:SetAllPoints(self)
end

function cellPrototype:SetupCell(tooltip, value, justification, font, iconCoords, unitID,guild)
	local tex = self.texture
	tex:SetWidth(16)
	tex:SetHeight(16)
	
	if guild then
		--Debug("SetSmallGuildTabardTextures")
		--SetSmallGuildTabardTextures("player", tex, tabard.background);
		SetSmallGuildTabardTextures("player", tex,tex);
	end
	
	if unitID then 
		SetPortraitTexture(tex, unitID)
	else
		tex:SetTexture(value)
	end
	if iconCoords then
		tex:SetTexCoord(unpack(iconCoords))
	end
	return tex:GetWidth(), tex:GetHeight() 
end

function cellPrototype:ReleaseCell()

end

-------------------------

local function UpdateText()
	local fps = floor(GetFramerate())
	local _, _, latency = GetNetStats()

	local fpsColor, latencyColor
	
	if fps > 30 then
		fpsColor = "|cff00ff00"
	elseif fps > 20 then
		fpsColor = "|cffffd200"
	else
		fpsColor = "|cffdd3a00"
	end
	if latency < 300 then
		latencyColor = "|cff00ff00"
		dataobj.icon = path.."green.tga"
	elseif latency < 500 then
		latencyColor = "|cffffd200"
		dataobj.icon = path.."yellow.tga"
	else
		latencyColor = "|cffdd3a00"
		dataobj.icon = path.."red.tga"
	end
	dataobj.text = string.format("%s%i|r ms %s%i|r fps", latencyColor, latency,fpsColor, fps )
end

local function MouseHandler(event, func, button, ...)
	--Debug("MouseHandler",event, func, button)
	local name = func
	if button == "RightButton" then
		--InviteUnit(name)
	else
		if type(func) == "function" then
			func(event, func,button, ...)
		else
			--Debug(func)
			func:GetScript("OnClick")(event, func,button, ...)
			--func:GetScript("OnMouseUp")(event, func,button, ...)
		end
	end
	LibQTip:Release(tooltip)
	tooltip = nil
end

function dataobj:OnEnter()
	if tooltip then 
		LibQTip:Release(tooltip) 
	end
	
	tooltip = LibQTip:Acquire("FooBarTooltip", 2, "LEFT", "LEFT")
	tooltip:Clear()
	self.tooltip = tooltip 

	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", myProvider, {0.2, 0.8, 0.2, 0.8},"player")
	tooltip:SetCell(y, 2, CharacterMicroButton.tooltipText)
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, function() ToggleCharacter("PaperDollFrame") end)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 1, path.."spells.tga", myProvider)
	tooltip:SetCell(y, 2, "Spellbook & Abilities |cffffd200(P)|r")
	--tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, SpellbookMicroButton)
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, function() 
		if InCombatLockdown() then
			local key = GetBindingKey("TOGGLESPELLBOOK")
			if key then
				DEFAULT_CHAT_FRAME:AddMessage("Cant' open the Spellbook during combat. Use your hot key: "..key)
			else
				DEFAULT_CHAT_FRAME:AddMessage("Cant' open the Spellbook during combat. Set and use a hot key.")
			end
		else
			if  SpellBookFrame:IsShown() then
				SpellBookFrame:Hide()
			else
				tinsert(UISpecialFrames,SpellBookFrame:GetName());
				SpellBookFrame:Show()
			end
		end
	end)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 1, path.."talents.tga", myProvider)
	tooltip:SetCell(y, 2, TalentMicroButton.tooltipText)
	--tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, TalentMicroButton)
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, function() 
		if InCombatLockdown() then
			local key = GetBindingKey("TOGGLETALENTS")
			if key then
				DEFAULT_CHAT_FRAME:AddMessage("Cant' open the Talents during combat. Use your hot key: "..key)
			else
				DEFAULT_CHAT_FRAME:AddMessage("Cant' open the Talents during combat. Set and use a hot key.")
			end
		else
			LoadAddOn("Blizzard_TalentUI")
			if  PlayerTalentFrame:IsShown() then
				PlayerTalentFrame:Hide()
			else
				tinsert(UISpecialFrames,PlayerTalentFrame:GetName());
				PlayerTalentFrame:Show()
			end
		end
	end)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 1, path.."achivements.tga", myProvider)
	tooltip:SetCell(y, 2, "Achievements")
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, AchievementMicroButton)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 1, path.."quest.tga", myProvider)
	tooltip:SetCell(y, 2, QuestLogMicroButton.tooltipText)
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, QuestLogMicroButton)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 1, path.."social.tga", myProvider)
	--tooltip:SetCell(y, 1, "", myProvider, nil,nil,true)
	tooltip:SetCell(y, 2, GuildMicroButton.tooltipText)
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, GuildMicroButton)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 1, path.."pvp.tga", myProvider)
	tooltip:SetCell(y, 2, PVPMicroButton.tooltipText)
	--tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, function() ToggleFrame(PVPParentFrame) end)
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, function() 
		--_, _, _, tocversion = GetBuildInfo()
		--if tocversion == "30350" then
		if PVPParentFrame then
			ToggleFrame(PVPParentFrame)
		else
			ToggleFrame(PVPFrame) 
		end
	end)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 1, path.."lfg.tga", myProvider)
	tooltip:SetCell(y, 2, LFDMicroButton.tooltipText)
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, LFDMicroButton)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 1, path.."green.tga", myProvider)
	tooltip:SetCell(y, 2, MainMenuMicroButton.tooltipText)
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, mainmenu)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 1, path.."help.tga", myProvider)
	tooltip:SetCell(y, 2, HelpMicroButton.tooltipText)
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, HelpMicroButton)
	
	tooltip:AddSeparator(10,0,0,0,0)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 1, "", myProvider)
	tooltip:SetCell(y, 2, GameMenuButtonOptions:GetText())
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, function() VideoOptionsFrame:Show() end)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 2, "Interface")
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, function() InterfaceOptionsFrame:Show() end)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 2, GameMenuButtonKeybindings:GetText())
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, function() KeyBindingFrame_LoadUI(); KeyBindingFrame:Show() end)
	
	local y, x = tooltip:AddLine()
	tooltip:SetCell(y, 2, GameMenuButtonMacros:GetText())
	tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, ShowMacroFrame)
	
	if GameMenuButtonAddOns then
		local y, x = tooltip:AddLine()
		tooltip:SetCell(y, 2, GameMenuButtonAddOns:GetText())
		tooltip:SetLineScript(y, "OnMouseUp", MouseHandler, GameMenuButtonAddOns)
	end
	
	tooltip:SetAutoHideDelay(0.01, self)
	tooltip:SmartAnchorTo(self)
	tooltip:Show()
end

local function OnUpdate(self, elapsed)
	counter = counter + elapsed
	if counter >= delay then
		UpdateText()
		counter = 0
	end
end

local frame = CreateFrame("Frame")
frame:SetScript("OnUpdate", OnUpdate)